@@ -4,7 +4,7 @@ use Moose;
 use namespace::autoclean;
 
 BEGIN {
-  extends 'Catalyst::Controller::ActionRole';
+  extends 'Catalyst::Controller';
 }
 
 __PACKAGE__->config(
